package interpreter;

import java.io.*;

import environment.RuntimeEnvironment;
import error.*;

public class LispInterpreterBuilderImpl implements LispInterpreterBuilder {

    private static LispInterpreterBuilder uniqueInstance = new LispInterpreterBuilderImpl();

    public static LispInterpreterBuilder getInstance() {
        return uniqueInstance;
    }

    private InputStream inputStream;
    private PrintStream outputStream;
    private PrintStream errorOutputStream;
    private Runnable terminationFunction;
    private Runnable errorTerminationFunction;
    private RuntimeEnvironment environment;
    private String fileName;
    private boolean isInteractive;
    private boolean isBuilt;

    private LispInterpreterBuilderImpl() {
        this.environment = RuntimeEnvironment.getInstance();
        this.fileName = "";
        this.isInteractive = true;
        this.isBuilt = false;
    }

    @Override
    public void setInput(InputStream inputStream) {
        this.inputStream = inputStream;
    }

    @Override
    public void setOutput(PrintStream outputStream) {
        this.outputStream = outputStream;

    }

    @Override
    public void setErrorOutput(PrintStream errorOutputStream) {
        this.errorOutputStream = errorOutputStream;
    }

    @Override
    public void setTerminationFunction(Runnable terminationFunction) {
        this.terminationFunction = terminationFunction;
    }

    @Override
    public void setErrorTerminationFunction(Runnable errorTerminationFunction) {
        this.errorTerminationFunction = errorTerminationFunction;
    }

    @Override
    public void useFile(String fileName) {
        this.fileName = fileName;
        this.isInteractive = false;
    }

    @Override
    public LispInterpreter build() {
        if (!isBuilt)
            return buildInterpreter();
        else
            throw new InterpreterAlreadyBuiltException();
    }

    private LispInterpreter buildInterpreter() {
        environment.setInput(inputStream);
        environment.setOutput(outputStream);
        environment.setErrorOutput(errorOutputStream);
        environment.setTerminationFunction(terminationFunction);
        environment.setErrorTerminationFunction(errorTerminationFunction);

        LispInterpreter lispInterpreter = isInteractive ? new InteractiveLispInterpreter()
                                                        : new LispInterpreter(fileName);

        isBuilt = true;

        return lispInterpreter;
    }

    public class InterpreterAlreadyBuiltException extends LispException {

        private static final long serialVersionUID = 1L;

        @Override
        public int getSeverity() {
            return ErrorManager.CRITICAL_LEVEL;
        }

        @Override
        public String getMessage() {
            return "Refusing to build more than one interpreter.";
        }
    }

}
